/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

/*
	Implements the broadcaster/listener design pattern
	See the Gamma book
*/
#ifndef ZzBroadcaster_h

#define ZzBroadcaster_h
#ifndef ZzBroadcaster_h
#include "ZzBroadcaster.h"
#endif //ZzBroadcaster_h

#ifndef ZzTypes_h
#include "ZzTypes.h"
#endif //ZzTypes_h

#ifndef ZzPtrArray_h
#include "ZzPtrArray.h"
#endif //ZzPtrArray_h

#ifndef ZzLocker_h
#include "ZzLocker.h"
#endif //ZzLocker_h

#ifndef ZzArray_h
#include "ZzArray.h"
#endif //ZzArray_h

/*

  predefs

*/

class ZzListener;

/*
	these classes implements the broadcaster/listener design pattern

    As soon as you need to pass information between two objects to notify 
	one of some event happening in the other and that you don't want to add 
	more and more member functions to the class, you basically derive your 
	info-provider from the ZzBroadcaster class.
	
	Then, you can either execute your event-handler in the same thread as
	the broadcaster events, or in another thread.

	For example, messages between elements of the GUI don't really need to
	happen in different classes, while events notified by the real-time 
	thread to the GUI thread must be asynchronous
	
	The object that will send messages needs to be a subclass of ZzBroadcaster
	An object that will receive synchronous notifications need to be a 
	subclass of MZzSyncListener, while an object expecting asynchronous 
	notifications need to be a subclass of MZzAsyncListener

	A synchronous notification is triggered in the NotificationReceived() method
	of an MZzSyncListener, while an asynchronous notification is a message
	sent to the thread associated with the MZzAsyncListener.
	Since we're using vfuncs or thread queue messages, we don't need to make 
	the classes known to each other, only a "MyMessages.h" file with codes is 
	enough.
*/
class ZzBroadcaster
{
protected :
			ZzBroadcaster(); //enforces derivation... useless stand-alone
public :
    void	Subscribe( ZzListener* listener );
    void	Unsubscribe( ZzListener* listener );
    void	Notify( UInt32 code, void* param = (void*)0); // notifies the listeners

private : 
    ZzArray<ZzListener*>	m_listeners;
    ZzRwLocker			m_listenerListMutex;
};

#endif //ZzBroadcaster_h

